
/////////////////////////////////////////////////////////////////////
// DESP-C++ (Discrete-Event Simulation Package)
// Version 1.1, February 1998
// Jerome Darmont
// LIMOS, Blaise Pascal University (Clermont-Ferrand II), France
/////////////////////////////////////////////////////////////////////
// eventc.h: Event Manager classes definition
// Variable with simulated systems
// => Flowshop sample (sample_eventc.h)
////////////////////////////////////////////////////////////////////////

// Active resources declaration

class Machine1;
class Machine2;

/////////////////////////////////////////////////////////////////////
// CLASS EventManager
/////////////////////////////////////////////////////////////////////
// Simulation events management
/////////////////////////////////////////////////////////////////////

// The Event Manager must "know" all (active and passive) resources.

class EventManager {

  public:

    // Methods

    EventManager(Simulation *sim);      // Constructor
    ~EventManager();                    // Destructor
    void ExecuteEvent(int code, Client *client); // Event execution
    void Init();                        // Initialization
    void InitRep();                     // Replication initialization 
    void Stats();                       // Stats computation (end of replication)
    void DisplayStats();                // Statistics display

  private:

    // Attributes

    Simulation *simul;                  // Pointer toward simulation object

    // Passive resource

    Resource *robot;

    // Active resources

    Machine1 *machine1;
    Machine2 *machine2;
};

/////////////////////////////////////////////////////////////////////
// CLASS Client
/////////////////////////////////////////////////////////////////////
// Custom simulation entity
/////////////////////////////////////////////////////////////////////

class Client {

  public:
           // Add here (in public) additional attributes

           Client *next;
           Client *previous;

};

/////////////////////////////////////////////////////////////////////
// CLASS Machine1
/////////////////////////////////////////////////////////////////////

class Machine1: public Resource {

  public:

    // Constructor

    Machine1(char n[STRS], int cap, Simulation *sim, Resource *rob, Machine2 *suiv);

    // Events

    void Event10(Client *client);       // Arrival on M1
    void Event11(Client *client);       // Begin product generation
    void Event12(Client *client);       // Product out
    void Event13(Client *client);       // Transfert toward M2

  private:                              // Pointers

    Resource *robot;                    // Robot (PR)
    Machine2 *m2;                       // Next machine (AR)

};

/////////////////////////////////////////////////////////////////////
// CLASS Machine2
/////////////////////////////////////////////////////////////////////

class Machine2: public Resource {

  public:

    // Constructor

    Machine2(char n[STRS], int cap, Simulation *sim, Resource *rob);

    // Events

    void Event20(Client *client);       // Arrival on M2
    void Event21(Client *client);       // Begin transformation
    void Event22(Client *client);       // End transformation
    void Event23(Client *client);       // Transfer out of M2
    void Event24(Client *client);       // Transfer out of system

  private:                              // Pointers

    Resource *robot;                    // Robot (PR)

    int production;                     // Counter

};
